#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using System;
using System.Windows.Forms;

namespace HeuristicLab.Optimization.Networks.Views {
  [View("Entity View")]
  [Content(typeof(Entity), true)]
  [Content(typeof(IEntity), false)]
  public partial class EntityView : NamedItemView {
    public new IEntity Content {
      get { return (IEntity)base.Content; }
      set { base.Content = value; }
    }

    public EntityView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      Content.PathChanged -= Content_PathChanged;
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.PathChanged += Content_PathChanged;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      toolTip.SetToolTip(nameTextBox, Content != null ? Content.Path : string.Empty);
    }

    protected virtual void Content_PathChanged(object sender, System.EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_PathChanged), sender, e);
      else {
        toolTip.SetToolTip(nameTextBox, Content != null ? Content.Path : string.Empty);
      }
    }
  }
}
