#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Collections;
using HeuristicLab.MainForm;
using HeuristicLab.Core.Views;
using HeuristicLab.Core;

namespace HeuristicLab.Optimization.Networks.Views {
  [View("MessageValueCollection View")]
  [Content(typeof(MessageValueCollection), true)]
  [Content(typeof(IKeyedItemCollection<string, IMessageValue>), false)]
  public partial class MessageValueCollectionView : ItemCollectionView<IMessageValue> {
    public new IKeyedItemCollection<string, IMessageValue> Content {
      get { return (IKeyedItemCollection<string, IMessageValue>)base.Content; }
      set { base.Content = value; }
    }

    public MessageValueCollectionView() {
      InitializeComponent();
      itemsGroupBox.Text = "Message Values";
    }

    protected override void DeregisterContentEvents() {
      Content.ItemsReplaced -= new CollectionItemsChangedEventHandler<IMessageValue>(Content_ItemsReplaced);
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.ItemsReplaced += new CollectionItemsChangedEventHandler<IMessageValue>(Content_ItemsReplaced);
    }

    protected override void OnContentChanged() {
      string selectedName = null;
      if ((itemsListView.SelectedItems.Count == 1) && (itemsListView.SelectedItems[0].Tag != null &&
        itemsListView.SelectedItems[0].Tag is IMessageValue))
        selectedName = ((IMessageValue)itemsListView.SelectedItems[0].Tag).Name;
      base.OnContentChanged();
      if (selectedName != null) {
        foreach (ListViewItem item in itemsListView.Items) {
          if ((item.Tag != null) && (((IMessageValue)item.Tag).Name.Equals(selectedName)))
            item.Selected = true;
        }
      }
    }

    #region Content Events
    protected virtual void Content_ItemsReplaced(object sender, CollectionItemsChangedEventArgs<IMessageValue> e) {
      if (InvokeRequired)
        Invoke(new CollectionItemsChangedEventHandler<IMessageValue>(Content_ItemsReplaced), sender, e);
      else {
        foreach (IMessageValue value in e.Items) {
          foreach (ListViewItem listViewItem in GetListViewItemsForItem(value))
            UpdateListViewItemText(listViewItem);
        }
        AdjustListViewColumnSizes();
      }
    }
    #endregion
  }
}
